@@ -4,11 +4,13 @@ import errno
 import functools
 import logging
 import socket
+import platform
 
-from nose.plugins.skip import SkipTest
+import pytest
 
-from urllib3.exceptions import MaxRetryError, HTTPWarning
+from urllib3.exceptions import HTTPWarning
 from urllib3.packages import six
+from urllib3.util import ssl_
 
 # We need a host that will not immediately close the connection with a TCP
 # Reset. SO suggests this hostname
@@ -29,10 +31,12 @@ def clear_warnings(cls=HTTPWarning):
         new_filters.append(f)
     warnings.filters[:] = new_filters
 
+
 def setUp():
     clear_warnings()
     warnings.simplefilter('ignore', HTTPWarning)
 
+
 def onlyPy26OrOlder(test):
     """Skips this test unless you are on Python2.6.x or earlier."""
 
@@ -40,10 +44,23 @@ def onlyPy26OrOlder(test):
     def wrapper(*args, **kwargs):
         msg = "{name} only runs on Python2.6.x or older".format(name=test.__name__)
         if sys.version_info >= (2, 7):
-            raise SkipTest(msg)
+            pytest.skip(msg)
         return test(*args, **kwargs)
     return wrapper
 
+
+def onlyPy34OrOlder(test):
+    """Skips this test unless you are on Python2.6.x or earlier."""
+
+    @functools.wraps(test)
+    def wrapper(*args, **kwargs):
+        msg = "{name} only runs on Python2.6.x or older".format(name=test.__name__)
+        if sys.version_info >= (3, 4):
+            pytest.skip(msg)
+        return test(*args, **kwargs)
+    return wrapper
+
+
 def onlyPy27OrNewer(test):
     """Skips this test unless you are on Python 2.7.x or later."""
 
@@ -51,10 +68,11 @@ def onlyPy27OrNewer(test):
     def wrapper(*args, **kwargs):
         msg = "{name} requires Python 2.7.x+ to run".format(name=test.__name__)
         if sys.version_info < (2, 7):
-            raise SkipTest(msg)
+            pytest.skip(msg)
         return test(*args, **kwargs)
     return wrapper
 
+
 def onlyPy279OrNewer(test):
     """Skips this test unless you are on Python 2.7.9 or later."""
 
@@ -62,10 +80,11 @@ def onlyPy279OrNewer(test):
     def wrapper(*args, **kwargs):
         msg = "{name} requires Python 2.7.9+ to run".format(name=test.__name__)
         if sys.version_info < (2, 7, 9):
-            raise SkipTest(msg)
+            pytest.skip(msg)
         return test(*args, **kwargs)
     return wrapper
 
+
 def onlyPy2(test):
     """Skips this test unless you are on Python 2.x"""
 
@@ -73,10 +92,11 @@ def onlyPy2(test):
     def wrapper(*args, **kwargs):
         msg = "{name} requires Python 2.x to run".format(name=test.__name__)
         if six.PY3:
-            raise SkipTest(msg)
+            pytest.skip(msg)
         return test(*args, **kwargs)
     return wrapper
 
+
 def onlyPy3(test):
     """Skips this test unless you are on Python3.x"""
 
@@ -84,17 +104,43 @@ def onlyPy3(test):
     def wrapper(*args, **kwargs):
         msg = "{name} requires Python3.x to run".format(name=test.__name__)
         if not six.PY3:
-            raise SkipTest(msg)
+            pytest.skip(msg)
         return test(*args, **kwargs)
     return wrapper
 
+
+def notSecureTransport(test):
+    """Skips this test when SecureTransport is in use."""
+
+    @functools.wraps(test)
+    def wrapper(*args, **kwargs):
+        msg = "{name} does not run with SecureTransport".format(name=test.__name__)
+        if ssl_.IS_SECURETRANSPORT:
+            pytest.skip(msg)
+        return test(*args, **kwargs)
+    return wrapper
+
+
+def onlyPy27OrNewerOrNonWindows(test):
+    """Skips this test unless you are on Python2.7+ or non-Windows"""
+    @functools.wraps(test)
+    def wrapper(*args, **kwargs):
+        msg = "{name} requires Python2.7+ or non-Windows to run".format(name=test.__name__)
+        if sys.version_info < (2, 7) and platform.system() == 'Windows':
+            pytest.skip(msg)
+        return test(*args, **kwargs)
+    return wrapper
+
+
 _requires_network_has_route = None
+
+
 def requires_network(test):
     """Helps you skip tests that require the network"""
 
     def _is_unreachable_err(err):
         return getattr(err, 'errno', None) in (errno.ENETUNREACH,
-                                               errno.EHOSTUNREACH) # For OSX
+                                               errno.EHOSTUNREACH)  # For OSX
 
     def _has_route():
         try:
@@ -121,7 +167,7 @@ def requires_network(test):
         else:
             msg = "Can't run {name} because the network is unreachable".format(
                 name=test.__name__)
-            raise SkipTest(msg)
+            pytest.skip(msg)
     return wrapper
 
 
